package be.iminds.ilabt.jfed.experimenter_gui.canvas.impl;

import be.iminds.ilabt.jfed.experimenter_gui.canvas.CanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.CanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.ExperimentCanvas;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.NewLinkTarget;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.EditableGtsDsl;
import be.iminds.ilabt.jfed.experimenter_gui.util.ImageUtil;
import be.iminds.ilabt.jfed.rspec.model.GeantTestbedType;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXGeantTestbedType;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecInterface;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import javafx.collections.MapChangeListener;
import javafx.geometry.Point2D;
import javafx.scene.image.Image;

import java.util.Map;

/**
 * User: twalcari
 * Date: 4/23/2015
 * Time: 15:21
 */
public class RspecGTSNode extends CanvasNode implements NewLinkTarget {

    private final FXGeantTestbedType gtt;
    private final EditableGtsDsl editableGtsDsl;

    public RspecGTSNode(FXGeantTestbedType gtt, ExperimentCanvas experimentCanvas) {
        super(experimentCanvas, gtt.getName(), new Image(ImageUtil.class.getResource("/images/gts-cloud.png").toExternalForm()));
        this.gtt = gtt;

        this.editableGtsDsl = new EditableGtsDsl(gtt.getName());
        editableGtsDsl.nameProperty().bindBidirectional(gtt.nameProperty());

        editableGtsDsl.dslProperty().addListener(observable -> {
            gtt.setDsl(editableGtsDsl.getDsl());
        });

        for (Map.Entry<FXRspecNode, FXRspecInterface> gtsToRspecBinding : editableGtsDsl.getAvailablePorts().entrySet()) {
            gtt.getPortBindings().put(gtsToRspecBinding.getKey().getClientId(),
                    new GeantTestbedType.RspecInterfaceBinding(gtsToRspecBinding.getValue().getNode().getClientId(),
                            gtsToRspecBinding.getValue().getClientId()));
        }

        editableGtsDsl.getAvailablePorts().addListener((MapChangeListener<FXRspecNode, FXRspecInterface>) change -> {
            if (change.wasAdded() && change.getValueAdded() != null) {
                gtt.getPortBindings().put(change.getKey().getClientId(),
                        new GeantTestbedType.RspecInterfaceBinding(change.getValueAdded().getNode().getClientId(),
                                change.getValueAdded().getClientId()));

            } else {
                gtt.getPortBindings().remove(change.getKey().getClientId());
            }
        });

        textProperty().bind(gtt.nameProperty());

        layoutXProperty().bindBidirectional(gtt.editorXProperty());
        layoutYProperty().bindBidirectional(gtt.editorYProperty());

        experimentCanvas.getCanvas().getChildren().add(this);
    }

    public FXGeantTestbedType getGeantTestbedType() {
        return gtt;
    }

    /**
     * Should be called when the RspecCanvasLink is removed from the canvas
     */
    public void remove() {
        experimentCanvas.getCanvas().getChildren().remove(this);
    }

    public EditableGtsDsl getEditableGtsDsl() {
        return editableGtsDsl;
    }

    @Override
    public Point2D getCenterPosition() {
        return new Point2D(CanvasLink.getCenterX(this), CanvasLink.getCenterY(this));
    }


}
