package be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.AbstractEditableCanvas;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.CanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.NewCanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.CanvasNodeFactory;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecCanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecCanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecUtils;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.views.DraggedItemProvider;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.GTSModelEditor;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox.GTSExternalNetworkToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox.GTSHostToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox.GTSOFXToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox.GTSPortToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.CustomChoiceDialog;
import be.iminds.ilabt.jfed.gts.GtsDslConstants;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.rspec.model.RspecInterface;
import be.iminds.ilabt.jfed.rspec.model.RspecNode;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecFactory;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.geometry.Point2D;
import javafx.scene.control.*;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.util.StringConverter;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Optional;

/**
 * User: twalcari
 * Date: 5/21/2015
 * Time: 10:28
 */
public class EditableGTSCanvas extends AbstractEditableCanvas {
    private static final Logger LOG = LoggerFactory.getLogger(EditableGTSCanvas.class);
    private final GTSModelEditor modelEditor;
    private final FXRspecFactory rspecFactory = FXRspecFactory.getInstance();

    private final ContextMenu nodeContextMenu;
    private final ContextMenu linkContextMenu;
    private final ContextMenu externalDomainContextMenu;

    public EditableGTSCanvas(GTSModelEditor modelEditor, JFedConfiguration config, CanvasNodeFactory canvasNodeFactory,
                             DraggedItemProvider draggedItemProvider, AuthorityList authorityList) {
        super(modelEditor.getModelRspec(), config, canvasNodeFactory, draggedItemProvider, authorityList);
        this.modelEditor = modelEditor;

        // create context menus
        this.nodeContextMenu = new ContextMenu();

        MenuItem configureNameItem =
                new MenuItem("Change node name", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.QUOTE_LEFT));
        configureNameItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

            onConfigureNodeNamePressed(rspecCanvasNode);
        });
        nodeContextMenu.getItems().add(configureNameItem);


        MenuItem configureItem = new MenuItem("Configure node", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
        configureItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

            onConfigureNodePressed(rspecCanvasNode);
        });
        nodeContextMenu.getItems().add(configureItem);

        MenuItem deleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
        deleteItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

            onDeleteNodePressed(rspecCanvasNode);
        });
        nodeContextMenu.getItems().add(deleteItem);

        //contextMenu for links
        this.linkContextMenu = new ContextMenu();

        // was disabled as FXRspecLink.LinkSetting is currently not supported in practice, but it seems to work for VM's now
        MenuItem configureLinkItem =
                new MenuItem("Configure link", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
        configureLinkItem.setOnAction(actionEvent -> {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

            onConfigureLinkPressed(rspecCanvasLink.getRspecLink());
        });
        linkContextMenu.getItems().add(configureLinkItem);

        MenuItem linkDeleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
        linkDeleteItem.setOnAction(actionEvent -> {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

            onDeleteLinkPressed(rspecCanvasLink);
        });
        linkContextMenu.getItems().add(linkDeleteItem);


        this.externalDomainContextMenu = new ContextMenu();

        MenuItem configureDomainItem =
                new MenuItem("Configure External Domain", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
        configureDomainItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedObject() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasExternalDomain = (RspecCanvasNode) getSelectionProvider().getSelectedObject();

            onConfigureExternalDomainPressed(rspecCanvasExternalDomain);
        });
        externalDomainContextMenu.getItems().add(configureDomainItem);

        MenuItem deleteDomainItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
        deleteDomainItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedObject() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasExternalDomain = (RspecCanvasNode) getSelectionProvider().getSelectedObject();

            onDeleteNodePressed(rspecCanvasExternalDomain);
        });
        externalDomainContextMenu.getItems().add(deleteDomainItem);

    }

    private void onConfigureNodeNamePressed(RspecCanvasNode rspecCanvasNode) {

        TextInputDialog dialog = new TextInputDialog(rspecCanvasNode.getRspecNode().getClientId());
        dialog.setHeaderText("Edit node name");
        dialog.setContentText("New node name:");

        dialog.showAndWait().ifPresent(newName -> {
            if (ModelRspecEditor.isValidNodeName(newName)) {
                rspecCanvasNode.getRspecNode().setClientId(newName);
            } else {
                new Alert(Alert.AlertType.ERROR, "The given name is invalid.").showAndWait();
            }
        });
    }

    private boolean onConfigureExternalDomainPressed(RspecCanvasNode rspecCanvasExternalDomain) {
        ChoiceDialog<ExternalDomain> dialog = new ChoiceDialog<>(domains[0], domains);

        dialog.setHeaderText("Select an external domain");
        dialog.setTitle("Select an external domain");

        Optional<ExternalDomain> result = dialog.showAndWait();
        if (result.isPresent()) {
            ExternalDomain externalDomain = result.get();
            rspecCanvasExternalDomain.getRspecNode().setComponentId(externalDomain.getLocationUrn());

            assert !rspecCanvasExternalDomain.getRspecNode().getInterfaces().isEmpty();

            rspecCanvasExternalDomain.getRspecNode().getInterfaces().get(0).setClientId(externalDomain.getPortId());
            return true;
        } else
            return false;
    }


    private static final ExternalDomain[] domains = new ExternalDomain[]{
            new ExternalDomain(GtsDslConstants.GTS_URN_LOCATION_AMS, "ep1", "iMinds #1", "from AMS via GN/BRU, BELnet", "Fed4FIRE\n"),
            new ExternalDomain(GtsDslConstants.GTS_URN_LOCATION_AMS, "ep2", "ONOS-US", "via GN/AMS, MANLAN, AL2S", "ONOS/DREAMER\n"),
            //    new ExternalDomain( "BRA","ep1","unassigned\n"+                                  ),
            new ExternalDomain(GtsDslConstants.GTS_URN_LOCATION_BRA, "ep2", "ExoGENI ", "via GN/AMS, MANLAN, AL2S", "GTS demo SC14\n"),
            new ExternalDomain(GtsDslConstants.GTS_URN_LOCATION_LJU, "ep1", "Create-net circuit #2 ", "via GN/MIL, GARR", "ONOS/DREAMER"),
            new ExternalDomain(GtsDslConstants.GTS_URN_LOCATION_LJU, "ep2", "iMinds #2", "from LJU via GN/BRU, BELnet", "Fed4FIRE"),
            //          new ExternalDomain("PRG","ep1","unassigned\n"),
            new ExternalDomain(GtsDslConstants.GTS_URN_LOCATION_PRG, "ep2", "Create-net circuit #1 ", "via GN/MIL, GARR", "ONOS/DREAMER")};

    private static class ExternalDomain {
        private final GeniUrn locationUrn;
        private final String portId;
        private final String friendlyName;
        private final String info1;
        private final String info2;

        public ExternalDomain(GeniUrn locationUrn, String portId, String friendlyName, String info1, String info2) {
            this.friendlyName = friendlyName;
            this.locationUrn = locationUrn;
            this.portId = portId;
            this.info1 = info1;
            this.info2 = info2;
        }

        public String getFriendlyName() {
            return friendlyName;
        }

        public String getInfo1() {
            return info1;
        }

        public String getInfo2() {
            return info2;
        }

        public GeniUrn getLocationUrn() {
            return locationUrn;
        }

        public String getPortId() {
            return portId;
        }

        public String toString() {
            return friendlyName + " (" + info1 + ")";
        }
    }

    private void onConfigureLinkPressed(FXRspecLink rspecLink) {
        TextInputDialog dialog = new TextInputDialog("10");
        dialog.setTitle("Link settings");
        dialog.setHeaderText("Link Settings");
        dialog.setContentText("Capacity [in Mbps]:");

        dialog.showAndWait().ifPresent(answer -> {
            boolean success = true;

            try {
                int value = Integer.parseInt(answer);
                if (value >= 1 && value <= 10_000) {
                    rspecLink.setSharedLan(answer);
                } else {
                    success = false;
                }
            } catch (NumberFormatException ex) {
                success = false;
            }

            if (!success) {
                new Alert(Alert.AlertType.ERROR, "Invalid capacity was entered! Only integer values between 1 and 10000 are allowed!").showAndWait();
            }
        });

    }

    private void onConfigureNodePressed(RspecCanvasNode rspecCanvasNode) {
        if (GtsDslConstants.GTS_URN_PORT_CM.equals(rspecCanvasNode.getRspecNode().getComponentManagerId())) {
            new Alert(Alert.AlertType.WARNING, "No settings for port yet.").showAndWait();
            return;
        }

        GeniUrn[] locations = new GeniUrn[]{GtsDslConstants.GTS_URN_LOCATION_BRA,
                                            GtsDslConstants.GTS_URN_LOCATION_AMS,
                                            GtsDslConstants.GTS_URN_LOCATION_LJU,
                                            GtsDslConstants.GTS_URN_LOCATION_PRG,
                                            GtsDslConstants.GTS_URN_LOCATION_HAM,
                                            GtsDslConstants.GTS_URN_LOCATION_MIL,
                                            GtsDslConstants.GTS_URN_LOCATION_LON};


        CustomChoiceDialog<GeniUrn> dialog = new CustomChoiceDialog<>(rspecCanvasNode.getRspecNode().getComponentId(), locations);
        dialog.setStringConverter(new StringConverter<GeniUrn>() {
            @Override
            public String toString(GeniUrn object) {
                return object.getEncodedResourceName();
            }

            @Override
            public GeniUrn fromString(String string) {
                return null;
            }
        });

        dialog.setContentText("Select the location for this host:");
        dialog.showAndWait().ifPresent(urn -> rspecCanvasNode.getRspecNode().setComponentId(urn));

    }

    @Override
    protected void deleteSelectedItem() {
        if (getSelectionProvider().getSelectedCanvasNode() != null) {
            CanvasNode canvasNode = getSelectionProvider().getSelectedCanvasNode();
            if (canvasNode instanceof RspecCanvasNode) {
                onDeleteNodePressed((RspecCanvasNode) canvasNode);
            }
        } else if (getSelectionProvider().getSelectedCanvasLink() != null) {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();
            onDeleteLinkPressed(rspecCanvasLink);
        }
    }

    @Override
    public boolean isLegalLink(NewCanvasLink newCanvasLink) {

        if (!newCanvasLink.isLinkComplete()) {
            LOG.trace("Requested link is not complete");
            return false;
        }

        if (newCanvasLink.getOrigin() instanceof RspecCanvasNode) {
            final FXRspecNode originNode = ((RspecCanvasNode) newCanvasLink.getOrigin()).getRspecNode();

            if (newCanvasLink.getTarget() instanceof RspecCanvasNode) {
                final FXRspecNode targetNode = ((RspecCanvasNode) newCanvasLink.getTarget()).getRspecNode();

                if (originNode == targetNode) {
                    LOG.trace("Origin and target-node cannot be identical.");
                    return false;
                } else {
                    return true;
                }
            } else if (newCanvasLink.getTarget() instanceof RspecCanvasLink) {
                final FXRspecLink targetLink = ((RspecCanvasLink) newCanvasLink.getTarget()).getRspecLink();

                //do not allow multiple interfaces to the same node to be added to a link
                if (targetLink.getInterfaceForNode(originNode) != null) {
                    LOG.trace("Node already has an interface on the requested link.");
                    return false;
                }
                return true;
            } else {
                throw new RuntimeException("Unexpected type of target node");
            }
        } else if (newCanvasLink.getOrigin() instanceof RspecCanvasLink) {
            FXRspecLink originLink = ((RspecCanvasLink) newCanvasLink.getOrigin()).getRspecLink();


            if (newCanvasLink.getTarget() instanceof RspecCanvasNode) {
                final FXRspecNode targetNode = ((RspecCanvasNode) newCanvasLink.getTarget()).getRspecNode();

                //do not allow multiple interfaces to the same node to be added to a link
                if (originLink.getInterfaceForNode(targetNode) != null) {
                    LOG.trace("Node already has an interface on the requested link.");
                    return false;
                }

                return true;
            } else if (newCanvasLink.getTarget() instanceof RspecCanvasLink) {
                //do not allow links between two RspecLinks
                LOG.trace("Cannot link two links together!");
                return false;
            } else {

                throw new RuntimeException("Unexpected type of target node");
            }
        } else {
            throw new RuntimeException("Unexpected type of origin node");
        }
    }


    private boolean canNodeAddLink(RspecNode node) {
        if (GtsDslConstants.GTS_URN_PORT_CM.equals(node.getComponentManagerId())) {
            return node.getInterfaces().isEmpty();
        }

        if (GtsDslConstants.GTS_URN_EXTERNAL_CM.equals(node.getComponentManagerId())) {
            RspecInterface tunnelIface = node.getInterfaceByClientId(GtsDslConstants.GTS_TUNNEL_ID);
            assert (tunnelIface != null);

            return tunnelIface.isLinkUnbound();
        }

        return true;
    }

    private void onDeleteNodePressed(RspecCanvasNode rspecCanvasNode) {
        model.deleteNode(rspecCanvasNode.getRspecNode());
        if (getSelectionProvider().getSelectedCanvasNode() == rspecCanvasNode) {
            getSelectionProvider().clearAllSelections();
        }

    }

    private void onDeleteLinkPressed(RspecCanvasLink rspecCanvasLink) {
        model.deleteLink(rspecCanvasLink.getRspecLink());
        if (getSelectionProvider().getSelectedCanvasLink() == rspecCanvasLink) {
            getSelectionProvider().clearAllSelections();
        }
    }

    @Override
    protected boolean processNewLinkRequest(NewCanvasLink newCanvasLink) {
        LOG.trace("Received new link request between {} AND {}", newCanvasLink.getOrigin(),
                newCanvasLink.getTarget());


        //delegate the real link-creation to ModelRspecEditor
        LOG.trace("Received link request is OK!");

        if (newCanvasLink.getOrigin() instanceof RspecCanvasNode && newCanvasLink.getTarget() instanceof RspecCanvasNode) {
            final FXRspecNode originNode = ((RspecCanvasNode) newCanvasLink.getOrigin()).getRspecNode();

            final FXRspecNode targetNode = ((RspecCanvasNode) newCanvasLink.getTarget()).getRspecNode();
            FXRspecLink newLink =
                    modelEditor.addLink(originNode, targetNode);

            setStatus("Created a new link '" + newLink.getClientId() + "'");
            return true;
        } else {
            setStatus("Can only create links between two nodes!");
            return false;
        }

    }

    @Override
    protected void addCanvasNodeFromToolboxItem(ToolboxItem item, Point2D location) {

        if (item instanceof GTSHostToolboxItem) {
            modelEditor.addHost(location);

        } else if (item instanceof GTSPortToolboxItem) {
            modelEditor.addPort(location);

        } else if (item instanceof GTSOFXToolboxItem) {
            modelEditor.addOFX(location);

        } else if (item instanceof GTSExternalNetworkToolboxItem) {
            FXRspecNode node = modelEditor.addExternalInterface(location);

            boolean success = onConfigureExternalDomainPressed(this.getRspecCanvasNode(node));

            if (!success) {
                model.deleteNode(node);
            }


        } else {
            throw new RuntimeException("Unexpected type of toolboxItem: " + item.getClass().getName());
        }
    }

    @Override
    protected RspecCanvasNode addCopyOfNodeToCanvas(RspecCanvasNode originalRspecCanvasNode) {

        FXRspecNode newRspecNode =
                rspecFactory.copyNode(model, originalRspecCanvasNode.getRspecNode(), RspecNode.InterfaceCopyMethod.NO_COPY);


        newRspecNode.setClientId(model.nextNodeName());
        newRspecNode.setEditorX(newRspecNode.getEditorX() + EDITOR_LOCATION_SHIFT);
        newRspecNode.setEditorY(newRspecNode.getEditorY() + EDITOR_LOCATION_SHIFT);

        model.getNodes().add(newRspecNode);

        return getRspecCanvasNode(newRspecNode);
    }

    public void arrangeNodesOnCircle() {
        arrangeNodesOnCircle(600, 400);
    }

    /**
     * This method arranges all the nodes on an ellipse of the given width and height
     */
    public void arrangeNodesOnCircle(double width, double height) {
        setZoom(1);
        ModelRspecUtils.arrangeNodesOnCircle(modelEditor.getModelRspec(), width, height);
    }

    @Override
    protected void canvasItemOnMouseClicked(MouseEvent mouseEvent) {
        super.canvasItemOnMouseClicked(mouseEvent);

        if (mouseEvent.getSource() instanceof RspecCanvasNode
                && !GtsDslConstants.GTS_URN_EXTERNAL_CM.equals(((RspecCanvasNode) mouseEvent.getSource()).getRspecNode().getComponentManagerId())) {

            RspecCanvasNode node = (RspecCanvasNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedCanvasNode(node);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureNodePressed(node);

            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                nodeContextMenu.show(node, mouseEvent.getScreenX(), mouseEvent.getScreenY());

            }
        }

        if (mouseEvent.getSource() instanceof RspecCanvasNode
                && GtsDslConstants.GTS_URN_EXTERNAL_CM.equals(((RspecCanvasNode) mouseEvent.getSource()).getRspecNode().getComponentManagerId())) {

            RspecCanvasNode node = (RspecCanvasNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedCanvasNode(node);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureExternalDomainPressed(node);

            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                externalDomainContextMenu.show(node, mouseEvent.getScreenX(), mouseEvent.getScreenY());

            }
        }

        if (mouseEvent.getSource() instanceof RspecCanvasLink.LinkCenter) {
            RspecCanvasLink link =
                    ((RspecCanvasLink.LinkCenter) mouseEvent.getSource()).getRspecCanvasLink();


            getSelectionProvider().setSelectedCanvasLink(link);
            assert (getSelectionProvider().getSelectedCanvasLink() != null);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureLinkPressed(link.getRspecLink());
            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                linkContextMenu.show(getSelectionProvider().getSelectedCanvasLink().getLinkCenter(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }
    }
}
