package be.iminds.jfed.gts_highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.util.JavaFXLogger;
import be.iminds.jfed.gts_highlevel.controller.GtsModel;
import be.iminds.jfed.gts_highlevel.controller.GtsReservationManagerFactory;
import be.iminds.jfed.gts_highlevel.controller.LoginControllerFactory;
import be.iminds.jfed.gts_highlevel.model.GtsProject;
import be.iminds.jfed.gts_highlevel.model.GtsReservation;
import be.iminds.jfed.gts_highlevel.model.GtsType;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.util.List;

/**
 * User: twalcari
 * Date: 10/8/2015
 * Time: 15:45
 */
@Singleton
public class GtsTaskFactory {
    private final LoginControllerFactory loginControllerFactory;
    private final GtsReservationManagerFactory gtsReservationManagerFactory;
    private final Provider<GtsLoginTask> loginGtsTaskProvider;
    private final GtsModel gtsModel;
    private final JavaFXLogger logger;
    private final TaskThread tt;

    @Inject
    public GtsTaskFactory(LoginControllerFactory loginControllerFactory, GtsReservationManagerFactory gtsReservationManagerFactory,
                          Provider<GtsLoginTask> loginGtsTaskProvider, GtsModel gtsModel, JavaFXLogger logger, TaskThread tt) {
        this.loginControllerFactory = loginControllerFactory;
        this.gtsReservationManagerFactory = gtsReservationManagerFactory;
        this.loginGtsTaskProvider = loginGtsTaskProvider;
        this.gtsModel = gtsModel;
        this.logger = logger;
        this.tt = tt;
    }

    public CreateGtsProjectTask createCreateGtsProjectTask(String projectName, String vpnUsername, String vpnPassword, List<String> users) {
        return new CreateGtsProjectTask(projectName, vpnUsername, vpnPassword, users,
                gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel, logger);
    }

    public DeleteGtsProjectTask createDeleteGtsProjectTask(GtsProject project) {
        return new DeleteGtsProjectTask(project, gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel, logger);
    }

    public StartGtsResourcesTask createStartGtsResourcesTask(String scriptName, String script, GtsProject project) {
        return new StartGtsResourcesTask(scriptName, script, project,
                gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel, logger);
    }

    public StopGtsResourcesTask createStopGtsResourcesTask(@Nonnull GtsReservation reservation, @Nonnull GtsType type, @Nonnull GtsProject project) {
        return new StopGtsResourcesTask(reservation, type, project,
                gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel, logger);
    }

    public UpdateGtsReservationsUntilReadyTask createUpdateGtsReservationsUntilReadyTask(GtsProject project, int reservationId){
        return new UpdateGtsReservationsUntilReadyTask(project, reservationId,
                gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel, logger,tt);
    }

    public GtsLoginTask createLoginGtsTask(){
        return new GtsLoginTask(loginControllerFactory, gtsModel, logger);
    }

    public GtsListResourcesTask createGtsListResourcesTask() {
        return new GtsListResourcesTask(logger, gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel);
    }

    public GtsListResourcesTask createGtsListResourcesTask(@Nonnull GtsProject project) {
        return new GtsListResourcesTask(project, logger, gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel);
    }
}
