package be.iminds.jfed.gts_highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.util.JavaFXLogger;
import be.iminds.jfed.gts_highlevel.controller.GtsException;
import be.iminds.jfed.gts_highlevel.controller.GtsModel;
import be.iminds.jfed.gts_highlevel.controller.GtsReservationManager;
import be.iminds.jfed.gts_highlevel.controller.GtsReservationManagerFactory;
import be.iminds.jfed.gts_highlevel.model.GtsProject;
import be.iminds.jfed.gts_highlevel.model.GtsReservation;
import be.iminds.jfed.gts_highlevel.model.GtsType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.inject.Provider;

/**
 * User: twalcari
 * Date: 9/25/2015
 * Time: 15:11
 */
public class StopGtsResourcesTask extends LoggedInGtsTask {
    private static final Logger LOG = LoggerFactory.getLogger(StopGtsResourcesTask.class);

    private final GtsReservation reservation;
    private final GtsType type;
    private final GtsProject project;

    StopGtsResourcesTask(@Nonnull GtsReservation reservation, @Nonnull GtsType type,@Nonnull GtsProject project,
                                 GtsReservationManagerFactory gtsReservationManagerFactory,
                                Provider<GtsLoginTask> loginGtsTaskProvider,
                                GtsModel gtsModel, JavaFXLogger logger) {
        super("Stop GTS Resources", logger, gtsReservationManagerFactory, loginGtsTaskProvider, gtsModel);
        this.reservation = reservation;
        this.type = type;
        this.project = project;
    }

    @Override
    protected void doGtsRMTask(TaskExecution taskExecution, GtsReservationManager reservationManager) throws GtsException {
        LOG.debug("Deactivating reservation {}", reservation.getId());
        reservationManager.deactivateReservation(reservation.getId());

        LOG.debug("Releasing reservation {}", reservation.getId());
        reservationManager.releaseReservation(reservation.getId());

        LOG.debug("Removing type {} from project {}", type.getName(), project.getId());
        reservationManager.removeType(project,type);

    }

    public GtsReservation getGtsReservation() {
        return reservation;
    }
}
