package be.iminds.ilabt.jfed.experiment.tasks;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;

/**
 * Differs from SingleCallExperimentTask in that it expects the task to be already submitted to the taskthread!
 *
 * User: twalcari
 * Date: 2/2/2015
 * Time: 14:24
 */
public class RunningCallExperimentTask<T extends Task> extends ExperimentTask {
    protected final T task;
    protected final TaskExecution<T> taskExecution;


    RunningCallExperimentTask(Experiment experiment,TaskExecution<T> taskExecution) {
        super(experiment);
        this.task = taskExecution.getTask();
        this.taskExecution = taskExecution;

    }

    public RunningCallExperimentTask(String title, Experiment experiment,TaskExecution<T> taskExecution) {
        super(experiment);
        this.task = taskExecution.getTask();
        this.taskExecution = taskExecution;

        updateTitle(title);
    }


    @Override
    protected final ExperimentTaskStatus executeTask() {
        while (!taskExecution.isCompleted()) {
            try {
                Thread.sleep(250);
            } catch (InterruptedException ignored) {
            }
        }

        registerFinishedTaskExecution(taskExecution);
        onTaskFinished(taskExecution);

        switch (taskExecution.getState()) {
            case SUCCESS:
                return ExperimentTaskStatus.SUCCESS;
            case FAILED:
            case CANCELLED:
                return ExperimentTaskStatus.FAILED;
            default:
                throw new IllegalStateException("Did not expected state " + taskExecution.getState().name()
                        + " for finished task " + task.toString());
        }
    }

    public void onTaskFinished(TaskExecution taskExecution) {
    }

    public TaskExecution getTaskExecution() {
        return taskExecution;
    }

    public T getTask() {
        return task;
    }

    @Override
    public String toString() {
        return getClass().getName() + "{" +
                "task=" + task.getName() +
                '}';
    }
}
