package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experiment.ExperimentPart;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.tasks.EditSSHKeysTask;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.function.Function;

/**
 * User: twalcari
 * Date: 11/27/2015
 * Time: 16:58
 */
public class EditSshKeysJob extends AbstractJob<Void> {
    private static final Logger LOG = LoggerFactory.getLogger(EditSshKeysJob.class);
    private final Experiment experiment;

    private final List<UserSpec> userSpecs;


    private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    private final JobStateFactory jobStateFactory;

    public EditSshKeysJob(Experiment experiment, List<UserSpec> userSpecs, HighLevelTaskFactory hltf, TaskThread tt,
                          AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                          JobStateFactory jobStateFactory) {
        super("Edit SSH keys in " + experiment.getName(), experiment, hltf, tt);
        this.experiment = experiment;
        this.userSpecs = userSpecs;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.jobStateFactory = jobStateFactory;
    }

    @Override
    public Void execute() throws Exception {
        setAndRunState(new EditSshKeysInExperimentPartsState());

        return null;
    }

    private class EditSshKeysInExperimentPartsState extends ExperimentPartsThreadedState<EditSshKeysInExperimentPartThread> {

        protected EditSshKeysInExperimentPartsState() {
            super("Edit SSH keys", experiment);
        }

        @Override
        public Function<ExperimentPart, EditSshKeysInExperimentPartThread> getExperimentPartToThreadFunction() {
            return EditSshKeysInExperimentPartThread::new;
        }
    }

    private class EditSshKeysInExperimentPartThread extends StatefulExperimentPartThread {

        public EditSshKeysInExperimentPartThread(ExperimentPart experimentPart) {
            super(experimentPart);
        }

        @Override
        public ExperimentTaskStatus statefulRun() throws JFedException, InterruptedException {
            if (aggregateManagerWrapperFactory.getAggregateManagerWrapper(experimentPart.getSfaAuthority())
                    .isEditSshKeysSupported()) {

                final EditSshKeysInExperimentPartState editSshKeysInExperimentPartState =
                        new EditSshKeysInExperimentPartState(experimentPart);
                setAndRunState(editSshKeysInExperimentPartState);

                if (editSshKeysInExperimentPartState.getTaskExecution().getState() == TaskExecution.TaskState.SUCCESS) {
                    setAndRunState(jobStateFactory.createGetStatusUntilReadyOrFailTaskState(EditSshKeysJob.this, experimentPart));
                } else {
                    LOG.warn("Updating SSH-keys failed in {}", experimentPart.getName());
                    return ExperimentTaskStatus.FAILED;
                }
            } else {
                LOG.info("Skipping updating SSH keys info on {}, as it is not supported", experimentPart.getSfaAuthority().getHrn());
                return ExperimentTaskStatus.WARNING;
            }
            return ExperimentTaskStatus.SUCCESS;
        }

    }

    private class EditSshKeysInExperimentPartState extends SingleCallState<EditSSHKeysTask> {
        protected EditSshKeysInExperimentPartState(ExperimentPart experimentPart) {
            super(String.format("Updating SSH keys on %s for %d user(s).", experimentPart.getName(), userSpecs.size()),
                    EditSshKeysJob.this, hltf.getEditSshKeysTask(experiment.getSlice(), experimentPart.getSfaAuthority(), userSpecs));
        }
    }

}
