package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experiment.ExperimentPart;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 11/25/2015
 * Time: 10:16
 */
public abstract class ExperimentPartsThreadedState<R extends StatefulExperimentPartThread> extends MultiThreadedState<R> {

    private final Experiment experiment;
    Map<ExperimentPart, R> experimentPartThreadMap = new HashMap<>();

    protected ExperimentPartsThreadedState(String name, Experiment experiment) {
        this(name, experiment, null);
    }

    /**
     * Use this constructor if you only want to execute the action for certain experiment parts.
     */
    protected ExperimentPartsThreadedState(String name, Experiment experiment, Collection<ExperimentPart> experimentParts) {
        super(name);
        this.experiment = experiment;

        Collection<ExperimentPart> parts = experimentParts != null ? experimentParts : experiment.getParts().values();

        experimentPartThreadMap = parts.stream()
                .collect(Collectors.toMap(Function.identity(), getExperimentPartToThreadFunction()));
    }


    @Override
    public Collection<R> getThreads() {
        return experimentPartThreadMap.values();
    }

    public abstract Function<ExperimentPart, R> getExperimentPartToThreadFunction();

    public Map<ExperimentPart, R> getExperimentPartThreads() {
        return experimentPartThreadMap;
    }
}
