package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experiment.ExperimentPart;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;

import javax.annotation.Nullable;
import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.util.List;

/**
 * User: twalcari
 * Date: 11/27/2015
 * Time: 17:12
 */
@Singleton
public class JobStateFactory {


    private final Provider<HighLevelTaskFactory> hltfProvider;
    private final Provider<AggregateManagerWrapperFactory> aggregateManagerWrapperFactoryProvider;
    private final GeniUserProvider geniUserProvider;
    private final Provider<EasyModel> easyModelProvider;

    @Inject
    public JobStateFactory(Provider<HighLevelTaskFactory> hltfProvider, Provider<AggregateManagerWrapperFactory> aggregateManagerWrapperFactoryProvider, GeniUserProvider geniUserProvider, Provider<EasyModel> easyModelProvider) {
        this.hltfProvider = hltfProvider;
        this.aggregateManagerWrapperFactoryProvider = aggregateManagerWrapperFactoryProvider;
        this.geniUserProvider = geniUserProvider;
        this.easyModelProvider = easyModelProvider;
    }

    public GetStatusUntilReadyOrFailTaskState createGetStatusUntilReadyOrFailTaskState(
            AbstractJob<?> job, ExperimentPart experimentPart) {

        return new GetStatusUntilReadyOrFailTaskState(job, experimentPart, hltfProvider.get());
    }

    public CreateUserSpecListState createCreateUserSpecListState(Experiment experiment) {
        return new CreateUserSpecListState(experiment, geniUserProvider, easyModelProvider.get());

    }


    public AllocateSliversOnAuthorityState createAllocateSliversOnAuthorityState(
            AbstractJob<?> job, ExperimentPart experimentPart, List<UserSpec> userSpecs,
            @Nullable RequestRspecSource overridingRequestRspecSource) {
        return new AllocateSliversOnAuthorityState(job, experimentPart, userSpecs, overridingRequestRspecSource,
                hltfProvider.get(), aggregateManagerWrapperFactoryProvider.get());
    }

    public AllocateSliversOnAuthorityState createAllocateSliversOnAuthorityState(
            AbstractJob<?> job, ExperimentPart experimentPart, List<UserSpec> userSpecs) {
        return createAllocateSliversOnAuthorityState(job, experimentPart, userSpecs, null);
    }

    public ProvisionSliversOnAuthorityState createProvisionSliversOnAuthorityState(
            AbstractJob<?> job, ExperimentPart experimentPart, List<UserSpec> userSpecs) {
        return new ProvisionSliversOnAuthorityState(job, experimentPart, userSpecs, hltfProvider.get());
    }

    public StopExperimentAtAuthorityPartState createStopExperimentAtAuthorityPartState(AbstractJob<?> job, ExperimentPart experimentPart) {
        return new StopExperimentAtAuthorityPartState(job, experimentPart, hltfProvider.get());
    }


}
