package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;

import java.util.Collection;

/**
 * When run, this class will start all the threads simultaneously, and then wait for each thread to finish
 */
public abstract class MultiThreadedState<R extends StatefulThread> extends State {

    protected MultiThreadedState(String name) {
        super(name);
    }

    public ExperimentTaskStatus executeState() throws InterruptedException {
        Collection<R> threads = getThreads();

        threads.forEach(R::start);

        for (R thread : threads) {
            thread.join();
        }

        boolean allSuccess = true;
        boolean allFailed = true;

        for (R thread : threads) {
            if (thread.getResult() != ExperimentTaskStatus.SUCCESS)
                allSuccess = false;
            if (thread.getResult() != ExperimentTaskStatus.FAILED)
                allFailed = false;
        }

        if (allSuccess)
            return ExperimentTaskStatus.SUCCESS;
        else if (allFailed)
            return ExperimentTaskStatus.FAILED;
        else
            return ExperimentTaskStatus.WARNING;
    }

    public abstract Collection<R> getThreads();

}
