package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.util.RFC3339Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * User: twalcari
 * Date: 7/15/2015
 * Time: 11:46
 */
public abstract class StartJob<V> extends AbstractJob<V> {
    private static final Logger LOG = LoggerFactory.getLogger(StartJob.class);

    protected final Experiment experiment;

    private final GeniUserProvider geniUserProvider;
    private final EasyModel easyModel;


    public StartJob(String name, Experiment experiment, HighLevelTaskFactory hltf, TaskThread tt, GeniUserProvider geniUserProvider, EasyModel easyModel) {
        super(name, experiment, hltf, tt);
        this.experiment = experiment;
        this.geniUserProvider = geniUserProvider;
        this.easyModel = easyModel;
    }

    @Nonnull
    @Override
    public Collection<Task> getDependsOn() {
        List<Task> res = new ArrayList<>();

        if (shouldRenewSlice())
            res.add(hltf.renewSlice(experiment.getSlice(), experiment.getRequestedEndTime()));

        res.add(hltf.getUserSSHKeys());
        if (experiment.isIncludeProjectMemberSshKeys())
            res.add(hltf.getSliceMembersSSHKeys(experiment.getSlice()));

        return res;
    }

    protected boolean shouldRenewSlice() {
        assert (experiment.getRequestedEndTime() != null);
        /*
        Note: when comparing times, take into account that the slice expiration date is set from the received date
         in RFC3339, which has no sub second precision.

         The requestedEndTime is truncated to second precision. So this is not an issue (anymore).

         see also FEDIBBTDEV-738

         */

        if (experiment.getSlice().getExpirationDate() != null) {
            if (experiment.getSlice().getExpirationDate().isBefore(experiment.getRequestedEndTime())) {
                LOG.info("Slice expires at " + experiment.getSlice().getExpirationDate() +
                        " (= " + RFC3339Util.dateToRFC3339String(Date.from(experiment.getSlice().getExpirationDate()), true, true, true) +
                        " = " + experiment.getSlice().getExpirationDate().toEpochMilli() + ")" +
                        " which is before the requested sliver expiration at " + experiment.getRequestedEndTime() +
                        " (" + RFC3339Util.dateToRFC3339String(Date.from(experiment.getRequestedEndTime()), true, true, true) +
                        " = " + experiment.getRequestedEndTime().toEpochMilli() + "). -> will update slice expiration");
                return true;
            } else {
                LOG.info("Slice expires at " + experiment.getSlice().getExpirationDate() +
                        " (= " + RFC3339Util.dateToRFC3339String(Date.from(experiment.getSlice().getExpirationDate()), true, true, true) +
                        " = " + experiment.getSlice().getExpirationDate().toEpochMilli() + ")" +
                        " which is NOT before the requested sliver expiration at " + experiment.getRequestedEndTime() +
                        " (" + RFC3339Util.dateToRFC3339String(Date.from(experiment.getRequestedEndTime()), true, true, true) +
                        " = " + experiment.getRequestedEndTime().toEpochMilli() + "). -> will NOT update slice expiration");
                return false;
            }
        } else {
            LOG.warn("Slice does not have an expiration date currently. We are suggesting a renewSlice to rectify this situation.");
            return true;
        }
    }


    protected static boolean supportsReservation(SfaAuthority auth) {
        return auth.hasSpecialCase(SfaAuthority.SpecialCase.SUPPORTS_RESERVATIONS_NITOS) ||
                auth.hasSpecialCase(SfaAuthority.SpecialCase.SUPPORTS_RESERVATIONS_NITOS);
    }

}
