package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.property.ReadOnlyObjectWrapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 11/25/2015
 * Time: 10:15
 */
public abstract class StatefulThread extends Thread {
    private static final Logger LOG = LoggerFactory.getLogger(StatefulThread.class);
    private final ReadOnlyObjectWrapper<be.iminds.ilabt.jfed.highlevel.jobs.State> jobState = new ReadOnlyObjectWrapper<>();

    private ExperimentTaskStatus result = null;


    @Override
    public final void run() {
        try {
            this.result = statefulRun();
        } catch (JFedException | InterruptedException e) {
            this.result = ExperimentTaskStatus.FAILED;
            LOG.error("UNCAUGHT exception in {}: {}", this.getClass().getName(), e.getMessage(), e);
        }
    }

    public abstract ExperimentTaskStatus statefulRun() throws JFedException, InterruptedException;

    protected void setAndRunState(be.iminds.ilabt.jfed.highlevel.jobs.State newState) throws InterruptedException, JFedException {
        this.jobState.set(newState);
        newState.run();
    }

    public be.iminds.ilabt.jfed.highlevel.jobs.State getJobState() {
        return jobState.get();
    }

    public ReadOnlyObjectProperty<be.iminds.ilabt.jfed.highlevel.jobs.State> jobStateProperty() {
        return jobState.getReadOnlyProperty();
    }

    public ExperimentTaskStatus getResult() {
        return result;
    }
}
