package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;

import java.util.Collections;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/16/2015
 * Time: 17:26
 */
public final class SliceStatusAtAuthorityTask extends Task {
    private final Slice slice;
    private final SfaAuthority auth;

    private StatusDetails status = null;

    private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    private final HighLevelTaskFactory highLevelTaskFactory;

    public SliceStatusAtAuthorityTask(Slice slice, SfaAuthority auth, AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                                      HighLevelTaskFactory highLevelTaskFactory) {
        super("Status of Slice @ " + auth.getName());
        this.slice = slice;
        this.auth = auth;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.highLevelTaskFactory = highLevelTaskFactory;
    }

    @Override
    public void doTask(TaskExecution taskExecution) throws JFedException, InterruptedException {
        AggregateManagerWrapper amWrapper =
                aggregateManagerWrapperFactory.getAggregateManagerWrapper(taskExecution, auth);

        if (!auth.hasAmUrl()) {
            throw new JFedHighLevelException("Tried to get Status for slice \"" + slice.getUrnString() + "\", " +
                    "at authority \"" + auth.getUrnString() + "\", but that authority has no AM URL: cannot continue.");
        }

        status = amWrapper.status(slice.getUrn(), slice.getCredentials());
        if (status == null)
            throw new JFedHighLevelException("Sliver status update failed");
    }

    @Override
    public List<Task> initDependsOn() {
        return Collections.singletonList(highLevelTaskFactory.getSliceCredential(slice));
    }

    public StatusDetails getStatus() {
        return status;
    }
}
